var WordCountPlus = {

	selectedWords: 0,
	totalWords: 0,
	previousTotal: -1,
	selectedChars: 0,
	totalChars: 0,
	previousCharTotal: -1,
	undoAllowed: false,
	autoCopy: false,
	autoDeselect: false,
	preCountText: "",
	postCountText: "",
	thousandSep: "",
	prefs: null,
	options: null,
	observerService: null,
	clipboardHelper: null,

	init: function() {	
		// Get all the prefs
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		this.getCounterState();
		

		// Get and monitor the preferences that are set in the options dialogue
		this.options = Components.classes["@mozilla.org/preferences-service;1"]
         										.getService(Components.interfaces.nsIPrefService)
		         										.getBranch("extensions.wordcountplus.options.");
		this.options.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this.options.addObserver("", this, false);
		this.getOptions();
		
		// Show Word Count Plus in the status bar
		this.updateStatus();
		
		// Watch for when the main area context menu is showing -- need to hide menu options if no text selected
		// SJ:20080709
		// Give the event listener a function with the correct context for "this".
		document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", function() { WordCountPlus.contextItemsHide(); }, false);
		
		// Watch for the application quit notification
		this.observerService = Components.classes["@mozilla.org/observer-service;1"]
	                                  .getService(Components.interfaces.nsIObserverService);
		this.observerService.addObserver(this, "quit-application-granted", false);
		
		// Set up access to the clipboard
		this.clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	},
	
	handleStatusClick: function(clickEvent) {
		if (clickEvent.button == 0 && clickEvent.shiftKey) {
			// on shift-left-click, count total words
			this.countTotalWords();
		}
		else if ((clickEvent.button == 1) || (clickEvent.button == 0 &&  clickEvent.altKey)) {
			// on middle click or alt-left-click, reset the counts 
			this.zeroCounts();
		}
		else if (clickEvent.button == 0) {
			// on left click, no key modifiers, count selected words
			this.countSelectedWords();
		}
	},
	
	openOptionsWindow: function() {
		// Only open the options window if it's not already open -- otherwise just focus it              
		var wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
                        .getService(Components.interfaces.nsIWindowMediator);
		var optionsWin = wm.getMostRecentWindow("mozilla:wordcountplus-options");
 		if (optionsWin) {
 			optionsWin.focus();
		}
		else {
			var instantApply = this.prefs.getBoolPref("browser.preferences.instantApply", false);
			var features = "chrome,titlebar,resizable=no,centerscreen" + (instantApply ? ",dialog=no" : "");
			window.openDialog("chrome://wordcountplus/content/options.xul", "", features);
		}
	},
		
	showPanel: function() {
		// When a window is activated, retrieve the current counter state and 
		// make the wordcount panel visible, hiding it in inactive panes
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator("navigator:browser");
		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			win.document.getElementById("wordcountplus-panel").collapsed = true;
		}
		this.getCounterState();
		this.updateStatus();
		document.getElementById("wordcountplus-panel").collapsed = false;		
	},
	
	doCount: function(sel) {
		var c = sel;
		if (c) {
			c = c.replace(/\s/g,' ').replace(/ {2,}/g,' ').replace(/ $|^ /g,'').split(' ');
			c = c.length;
		}
		else {
			alert(document.getElementById("wordcountplus-strings").getString("nothingSelectedMessage"))
			c = 0;
		}
		return c;
	},
	
	getSelectedText: function() {
		var c = document.commandDispatcher.focusedWindow.getSelection().toString();
		if (c) {
			return c;	
		}
		else {
			// Nothing selected in the window, try for a text box
			var e = document.commandDispatcher.focusedElement;
			if (e) {
				if ((e.tagName.toLowerCase() == "input" && (e.type == "text")) || (e.tagName.toLowerCase() == "textarea") || (e.tagName.toLowerCase() == "textbox")) {
					var st = e.selectionStart;
					var end = e.selectionEnd;
					if (end > st) {
						c = e.value.substring(st, end);
						return c;
					}
				}
			}
		}
		return "";	
	},
	
	collapseSelected: function() {
		var c = document.commandDispatcher.focusedWindow.getSelection().toString();
		if (c) {
			document.commandDispatcher.focusedWindow.getSelection().collapseToStart();
		}
		else {
			// Nothing selected in the window, try for a text box
			var e = document.commandDispatcher.focusedElement;
			if (e) {
				if ((e.tagName.toLowerCase() == "input" && (e.type == "text")) || (e.tagName.toLowerCase() == "textarea") || (e.tagName.toLowerCase() == "textbox")) {
					e.setSelectionRange(e.selectionStart, e.selectionStart);
				}
			}
		}	
	},
	
	countSelectedWords: function() {	
		var sel = this.getSelectedText();
		var c = this.doCount(sel);
		if (c != 0) {
			this.selectedWords = c;
			this.selectedChars = sel.length;
			this.prefs.setIntPref("extensions.wordcountplus.selectedWords", this.selectedWords);
			this.prefs.setIntPref("extensions.wordcountplus.selectedChars", this.selectedChars);
			this.updateStatus();
			
			if (this.autoCopy) {
				this.doCopy(this.selectedWords);
			}
			
			if (this.autoDeselect) {
				// deselect the selection		
				this.collapseSelected();	
			}
		}
	},
	
	countTotalWords: function() {
		var existingWordCount = 0;
		var existingCharCount = 0;
		if (this.totalWords == 0) {
			existingWordCount = this.selectedWords;		
			existingCharCount = this.selectedChars;		
		}	
		var sel = this.getSelectedText();
		var c = this.doCount(sel);
		if (c != 0) {
			this.selectedWords = c;
			this.previousTotal = this.totalWords;
			this.totalWords = this.totalWords + c + existingWordCount;
			this.selectedChars = sel.length;
			this.previousCharTotal = this.totalChars;
			this.totalChars= this.totalChars + this.selectedChars + existingCharCount;
			this.undoAllowed = true;
			this.prefs.setIntPref("extensions.wordcountplus.selectedWords", this.selectedWords);
			this.prefs.setIntPref("extensions.wordcountplus.totalWords", this.totalWords);
			this.prefs.setIntPref("extensions.wordcountplus.previousTotal", this.previousTotal);
			this.prefs.setIntPref("extensions.wordcountplus.selectedChars", this.selectedChars);
			this.prefs.setIntPref("extensions.wordcountplus.totalChars", this.totalChars);
			this.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", this.previousCharTotal);
			this.prefs.setBoolPref("extensions.wordcountplus.undoAllowed", this.undoAllowed);
			this.updateStatus();
			
			if (this.autoCopy) {
				this.doCopy(this.totalWords);
			}
			
			if (this.autoDeselect) {
				// deselect the selection	
				this.collapseSelected();			
			}
		}
	},
	
	zeroCounts: function() {
		if (this.selectedWords != 0 || this.totalWords != 0) {
			if (this.selectedWords != 0 ) {
				this.selectedWords = 0;
				this.selectedChars = 0;
				this.prefs.setIntPref("extensions.wordcountplus.selectedWords", this.selectedWords);
				this.prefs.setIntPref("extensions.wordcountplus.selectedChars", this.selectedChars);
			}
			if (this.totalWords != 0) {
				this.previousTotal = this.totalWords;
				this.totalWords = 0;
				this.previousCharTotal = this.totalChars;
				this.totalChars = 0;
				this.undoAllowed = true;
				this.prefs.setIntPref("extensions.wordcountplus.totalWords", this.totalWords);
				this.prefs.setIntPref("extensions.wordcountplus.previousTotal", this.previousTotal);
				this.prefs.setIntPref("extensions.wordcountplus.totalChars", this.totalChars);
				this.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", this.previousCharTotal);
				this.prefs.setBoolPref("extensions.wordcountplus.undoAllowed", this.undoAllowed);
			}
			this.updateStatus();
		}
	},
	
	undoredoCounter: function() {
		if (this.previousTotal >= 0) {
			var currentTotal = this.totalWords;
			var currentCharTotal = this.totalChars;
			this.totalWords = this.previousTotal;
			this.previousTotal = currentTotal;
			this.totalChars = this.previousCharTotal;
			this.previousCharTotal = currentCharTotal;
			this.undoAllowed = !this.undoAllowed;
			this.prefs.setIntPref("extensions.wordcountplus.totalWords", this.totalWords);
			this.prefs.setIntPref("extensions.wordcountplus.previousTotal", this.previousTotal);
			this.prefs.setIntPref("extensions.wordcountplus.totalChars", this.totalChars);
			this.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", this.previousCharTotal);
			this.prefs.setBoolPref("extensions.wordcountplus.undoAllowed", this.undoAllowed);
			this.updateStatus();
		}
	},
	
	updateStatus: function() {
		var wordStatus = document.getElementById("wordcountplus-panel");
		var wordtext = "";
		if (this.selectedWords != 0) {
			wordtext = "words";
			switch (this.selectedWords) {
				case 1:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsOne");
				 break;
				case 2:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsTwo");
				 break;
				default:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsMore");
			}
			wordtext = this.addThousandSep(this.selectedWords) + ' ' + wordtext;
		}
		if (this.totalWords != 0) {			
			var totaltext = document.getElementById("wordcountplus-strings").getString("totalWords");
			wordtext = (wordtext != "") ? wordtext + ' | ' : wordtext;
			wordtext = wordtext + this.addThousandSep(this.totalWords) + ' ' + totaltext;
		}
		
		// Need to change the style of the panel depending on whether there is text or not, to 
		// get nice even spacing in all cases.
		if (wordtext == "") {
			wordStatus.setAttribute("class", "statusbarpanel-iconic");
			wordStatus.setAttribute("src", "chrome://wordcountplus/skin/wcp-small.png");	
		}
		else {
			wordStatus.setAttribute("class", "statusbarpanel-iconic-text");
			wordStatus.setAttribute("src", "chrome://wordcountplus/skin/wcp-small.png");
		}
		wordStatus.label = wordtext;
	},
	
	updateTooltip: function() {
		var chartext = "";
		if (this.selectedChars != 0) {
			chartext = "chars";
			switch (this.selectedChars) {
				case 1:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsOne");
				 break;
				case 2:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsTwo");
				 break;
				default:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsMore");
			}
			chartext = this.addThousandSep(this.selectedChars) + ' ' + chartext;
		}
		if (this.totalChars != 0) {			
			var totaltext = document.getElementById("wordcountplus-strings").getString("totalChars");
			chartext = (chartext != "") ? chartext + ' | ' : chartext;
			chartext = chartext + this.addThousandSep(this.totalChars) + ' ' + totaltext;
		}
		if (chartext == "") {
			chartext = document.getElementById("wordcountplus-strings").getString("wcpName");			
		}
		document.getElementById("wordcountplus-tooltiptext").setAttribute("value", chartext);
	},
	
	updateStatusContext: function() {		
		// Count Actions
		var sel = this.getSelectedText();
		var noSelection = (sel == "") ? true : false;
		document.getElementById("wordcountplus-statuscount").setAttribute("disabled", noSelection);
		document.getElementById("wordcountplus-statusadd").setAttribute("disabled", noSelection);
		
		// Undo-Redo
		var undoMenuItem = document.getElementById("wordcountplus-undo");
		var redoMenuItem = document.getElementById("wordcountplus-redo");
		if (this.previousTotal < 0) {
			undoMenuItem.setAttribute("disabled", "true");
			redoMenuItem.setAttribute("disabled", "true");
		}
		else {
			if (this.undoAllowed) {
				undoMenuItem.setAttribute("disabled", "false");
				redoMenuItem.setAttribute("disabled", "true");
			}
			else {
				undoMenuItem.setAttribute("disabled", "true");
				redoMenuItem.setAttribute("disabled", "false");
			}
		}		
		
		// Reset to Zero
		var resetMenuItem = document.getElementById("wordcountplus-reset");
		if (this.totalWords == 0 && this.selectedWords == 0) {
			resetMenuItem.setAttribute("disabled", "true");
		}
		else {		
			resetMenuItem.setAttribute("disabled", "false");
		}		
		
		// Copy Words
		var copyselectedMenuItem = document.getElementById("wordcountplus-copyselected");
		if (this.selectedWords == 0) {
			copyselectedMenuItem.setAttribute("disabled", "true");
		}
		else {		
			copyselectedMenuItem.setAttribute("disabled", "false");
		}	
		
		// Copy Total
		var copytotalMenuItem = document.getElementById("wordcountplus-copytotal");
		if (this.totalWords == 0) {
			copytotalMenuItem.setAttribute("disabled", "true");
		}
		else {		
			copytotalMenuItem.setAttribute("disabled", "false");
		}	
		
		// Auto Copy
		var autocopyMenuItem = document.getElementById("wordcountplus-autocopy");
		autocopyMenuItem.setAttribute("checked", this.autoCopy);
		
		// Auto Deselect
		var autodeselectMenuItem = document.getElementById("wordcountplus-autodeselect");
		autodeselectMenuItem.setAttribute("checked", this.autoDeselect);
	},
		
	contextItemsHide: function() {
		var sel = this.getSelectedText();
		var noSelection = (sel == "") ? true : false;
		document.getElementById("wordcountplus-count").hidden = noSelection;
		document.getElementById("wordcountplus-add").hidden = noSelection;
	},
		
	toggleAutoCopy: function(e) {
		e.preventDefault();
		e.stopPropagation();
		var ac = this.autoCopy; 
		document.getElementById("wordcountplus-autocopy").checked = !ac;
		this.autoCopy = !ac;
		this.prefs.setBoolPref("extensions.wordcountplus.autoCopy", !ac);		
	},
	
	toggleAutoDeselect: function(e) {
		e.preventDefault();
		e.stopPropagation();
		var ads = this.autoDeselect;
		document.getElementById("wordcountplus-autodeselect").checked = !ads;
		this.autoDeselect = !ads;
		this.prefs.setBoolPref("extensions.wordcountplus.autoDeselect", !ads);		
	},
	
	getCounterState: function() {	
		this.selectedWords = 0;		
		this.totalWords = 0;
		this.previousTotal = -1;
		this.selectedChars = 0;		
		this.totalChars = 0;
		this.previousCharTotal = -1;
		this.undoAllowed = false;
		this.autoCopy = false;
		this.autoDeselect = false;
		
		try {	
			this.selectedWords = this.prefs.getIntPref("extensions.wordcountplus.selectedWords");
			this.totalWords = this.prefs.getIntPref("extensions.wordcountplus.totalWords");
			this.previousTotal = this.prefs.getIntPref("extensions.wordcountplus.previousTotal");
			this.selectedChars = this.prefs.getIntPref("extensions.wordcountplus.selectedChars");
			this.totalChars = this.prefs.getIntPref("extensions.wordcountplus.totalChars");
			this.previousCharTotal = this.prefs.getIntPref("extensions.wordcountplus.previousCharTotal");
			this.undoAllowed = this.prefs.getBoolPref("extensions.wordcountplus.undoAllowed");			
			this.autoCopy = this.prefs.getBoolPref("extensions.wordcountplus.autoCopy");			
			this.autoDeselect = this.prefs.getBoolPref("extensions.wordcountplus.autoDeselect");
		}
		catch(e) {}
	},
		
	getOptions: function() {
		this.preCountText = "";
		this.postCountText = "";
		this.thousandSep = "";
    	
		try {
			this.preCountText = this.options.getCharPref("textBeforeCount");
			this.postCountText = this.options.getCharPref("textAfterCount");
			this.thousandSep = this.options.getCharPref("thousandSep");
		}
		catch(e) {}
	},
	
	doCopy: function(numWords) {
		var numString = this.customFormat(numWords);
		this.clipboardHelper.copyString(numString);
	},
	
	customFormat: function(numWords) {
		var numString = numWords;
		if (numString != "") {
			numString = this.addThousandSep(numWords);
			numString = this.preCountText + numString + this.postCountText;
		}
		return numString;
	},
	
	addThousandSep: function(numWords) {
		var numString = numWords.toString();
		if ((numString != "") && (this.thousandSep != "")) {
			numString = numString.replace(/(\d)(?=(\d{3})+([.]|$))/g,"$1"+this.thousandSep);
		}
		return numString;
	},
	
	observe: function(subject, topic, data) {
		// Respond to pertinent notifications
		switch(topic) {
			case "nsPref:changed":
			// The preferences were changed in the Options dialogue
			this.getOptions();
			this.updateStatus();
			break;
			
			case "quit-application-granted":
			// Reset counts to zero and disable undo functionality
			this.prefs.setIntPref("extensions.wordcountplus.selectedWords", 0);
			this.prefs.setIntPref("extensions.wordcountplus.totalWords", 0);
			this.prefs.setIntPref("extensions.wordcountplus.previousTotal", -1);
			this.prefs.setIntPref("extensions.wordcountplus.selectedChars", 0);
			this.prefs.setIntPref("extensions.wordcountplus.totalChars", 0);
			this.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", -1);
			this.observerService.removeObserver(wcp_quitObserver, "quit-application-granted");
			break;
			
			default:
			// do nothing
		}
	}

};

// SJW:20080709
// Ah, the joys of revisiting one's first extension. Switched over to using keyword this
// in the WordCountPlus var, and moved these event-triggered calls into functions so they
// get the right context (otherwise "this" points to the window, not the WCP object).
window.addEventListener("load", function() { WordCountPlus.init();}, false);
window.addEventListener("focus", function() { WordCountPlus.showPanel();}, false);	